#include "Includes.h"

void Init_Timer0(void);
void Init_Timer2(void);
void Init_Interrupt(void);
void Init_ADC(void);


volatile unsigned int Drehzahl = 0;
volatile unsigned int Geschwindigkeit = 0;
volatile unsigned int ADCWert;
volatile unsigned int cnt_helligkeit = 0;
volatile unsigned int Umfang = 200;
volatile unsigned int rpm_cnt;
volatile unsigned int rpm;
volatile unsigned int ms_cnt;
volatile unsigned int ms;

int x = 0;
int main(void)
{
	Init_UART();
	Init_I2C();
	Init_LCD();
	Init_Timer0();
	Init_Timer2();
	Init_Interrupt();
	Init_SR();
	Init_ADC();
	
	DDRD |= (1<<PD7);		//PWM LED Helligkeit


	DS1307Write(0x00,0x00);
	_delay_ms(1);
	DS1307Write(0x01,0x05);
	_delay_ms(1);
	DS1307Write(0x02,0x19);
	_delay_ms(1);
	DS1307Write(0x03,0x00);
	_delay_ms(1);
	DS1307Write(0x04,0x08);
	_delay_ms(1);
	DS1307Write(0x05,0x01);
	_delay_ms(1);
	DS1307Write(0x06,0x12);
	_delay_ms(1);
	DS1307Write(0x07,0x00);
	_delay_ms(1);
	sei();
	
	
	put_string("km/h", 74, 0, font_12x16);
	put_string("TAG: ", 40, 16, font_6x8);
	put_string("GES: ", 40, 24, font_6x8);


	while(1)
    {	
	
		if(rpm != 0 && rpm_cnt < 1500)
			Drehzahl = 600000/rpm;
		else
			Drehzahl = 0;

		
		if(ms != 0 && ms_cnt < 15000)
			Geschwindigkeit = (Umfang*36)/(0.2*ms);
		else
			Geschwindigkeit = 0;
		
		
		put_number(Geschwindigkeit, 0, 0, 3, 0, font_12x16);
		put_number(0, 71, 16, 3, 0, font_6x8);
		put_number(0, 71, 24, 3, 0, font_6x8);
		put_number(OCR2, 40, 32, 3, 0, font_6x8);

		put_time(40, 55);

		sr_put(0xFFFF>>(16-Drehzahl/562));
		_delay_ms(10);
		ADCSRA |= (1<<ADSC);
	
	
    }
}



void Init_ADC(void)
{
	ADMUX = 0x01;
	ADCSRA = 0x88;	
}



void Init_Timer0(void)	//Timer fr MS und RPM
{
	cli();
	TCCR0 = 0x0A;		//Timer2 Prescale 64, CTC Mode
	OCR0 = 200;			//Vergleichswert von 200
	TIMSK = 0x02;		//Interrupt Freigabe
}



void Init_Timer2(void)	//Timer fr PWM, kein Interrupt ntig, OCR2 wird ber ADC eingestellt
{
	cli();
	TCCR2 = 0x6C;		//Timer2 Fast-PWM, Prescale von 64
	OCR2 = 30;			//Vergleichswert von 30
}




void Init_Interrupt(void)
{
	cli();
	MCUCR = 0x0E;		//Steigende Flanke an INT0/INT1 lst Interrupt aus
	GICR = 0xC0;		//INT0 und INT1 Interrupt enable
}




ISR(INT0_vect)
{
	rpm = rpm_cnt;
	rpm_cnt = 0;	
}



ISR(INT1_vect)
{
	ms = ms_cnt;
	ms_cnt = 0;	
}





ISR(TIMER0_COMP_vect)
{
	
	if(rpm_cnt < 1600)
		rpm_cnt++;
	
	if(ms_cnt < 16000)
		ms_cnt++;

	if(ms < ms_cnt)
		ms = ms_cnt;			
}



ISR(ADC_vect)
{	
	
	if(ADCW/4  > (OCR2 + 10))
		OCR2 += 1;
	else if(OCR2 > 30)
		OCR2 -= 1;
}








